﻿------------------------------------------------------------------------------
--	Valet -- (C) 2009-2010 Kyahx.  All Rights Reserved
--
--	Most of this code was inspired by (or shamelessly copied from)
--	Tekkub's mods, Kennel and Buffet. All credit to him where credit is due.
------------------------------------------------------------------------------

local MAX_ACCOUNT_MACROS, MAX_CHARACTER_MACROS = 36, 18

local NUMROWS, NUMCOLS, ICONSIZE, GAP, EDGEGAP = 5, 8, 32, 8, 16
local rows = {}

local frame = CreateFrame('Frame', nil, InterfaceOptionsFramePanelContainer)
frame.name = 'Valet'
frame:Hide()
frame:SetScript('OnShow', function(frame)
	local tektab = LibStub('tekKonfig-TopTab')
	
	local title, subtitle = LibStub('tekKonfig-Heading').new(frame, 'Valet', 'This panel allows you to select which mounts Valet will use.  You can also modify the base macro for Valet to edit.')

	local function OnClick(self)
		local id = GetMacroIndexByName(self.name)
		if id and id ~= 0 then PickupMacro(id)
		elseif GetNumMacros() >= MAX_ACCOUNT_MACROS then Valet:Print('All global macros in use.')
		else
			local id = CreateMacro(self.name, 1, '')
			Valet:Shuffle()
			PickupMacro(id)
		end
	end

	local macroButton = LibStub('tekKonfig-Button').new(frame, 'TOPLEFT', subtitle, 'BOTTOMLEFT', -2, -GAP)
	macroButton:SetText('Macro')
	macroButton.tiptext = 'Generate a global macro for mounting.'
	macroButton.name = 'AutoMount'
	macroButton:SetScript('OnClick', OnClick)
	if InCombatLockdown() then macroButton:Disable() end

	frame:SetScript('OnEvent', function(self, event)
		if event == 'PLAYER_REGEN_DISABLED' then macroButton:Disable() else macroButton:Enable() end
	end)
	frame:RegisterEvent('PLAYER_REGEN_ENABLED')
	frame:RegisterEvent('PLAYER_REGEN_DISABLED')

	local group = LibStub('tekKonfig-Group').new(frame, nil, 'TOP', macroButton, 'BOTTOM', 0, -GAP-32)
	group:SetPoint('LEFT', EDGEGAP, 0)
	group:SetPoint('BOTTOMRIGHT', -EDGEGAP, EDGEGAP)

	local mountFrame, macroFrame
	local groundTab = tektab.new(frame, 'Ground', 'BOTTOMLEFT', group, 'TOPLEFT', 0, -4)
	local flyingTab = tektab.new(frame, 'Flying', 'LEFT', groundTab, 'RIGHT', -15, 0)
	local macroTab = tektab.new(frame, 'Macro', 'LEFT', flyingTab, 'RIGHT', -15, 0)
	flyingTab:Deactivate()
	macroTab:Deactivate()
	groundTab:SetScript('OnClick', function(self)
		self:Activate()
		flyingTab:Deactivate()
		macroTab:Deactivate()
		mountFrame.type = 'ground'
		mountFrame:Update()
		macroFrame:Hide()
	end)
	flyingTab:SetScript('OnClick', function(self)
		self:Activate()
		groundTab:Deactivate()
		macroTab:Deactivate()
		mountFrame.type = 'flyer'
		mountFrame:Update()
		macroFrame:Hide()
	end)
	macroTab:SetScript('OnClick', function(self)
		self:Activate()
		groundTab:Deactivate()
		flyingTab:Deactivate()
		mountFrame:Hide()
		macroFrame:Show()
	end)


	---------------------------
	--      Mount panel      --
	---------------------------
	
	mountFrame = CreateFrame('Frame', nil, group)
	mountFrame:SetAllPoints()
	mountFrame.type = 'ground'

	local scrollbar = LibStub('tekKonfig-Scroll').new(mountFrame, 6, 1)

	mountFrame:EnableMouseWheel()
	mountFrame:SetScript('OnMouseWheel', function(self, val) scrollbar:SetValue(scrollbar:GetValue() - val) end)

	local mountFrameText = mountFrame:CreateFontString(nil, 'ARTWORK', 'GameFontHighlightSmall')
	mountFrameText:SetHeight(32)
	mountFrameText:SetPoint('TOPLEFT', mountFrame, 'TOPLEFT', EDGEGAP, -EDGEGAP)
	mountFrameText:SetPoint('RIGHT', mountFrame, -EDGEGAP-16, 0)
	mountFrameText:SetNonSpaceWrap(true)
	mountFrameText:SetJustifyH('LEFT')
	mountFrameText:SetJustifyV('TOP')
	
	local function OnClick(self)
		local index = Valet.db[mountFrame.type][self.name]
		if index then
			tremove(Valet.db[mountFrame.type], index)
		else
			tinsert(Valet.db[mountFrame.type], self.name)
		end
		self:GetNormalTexture():SetDesaturated(Valet.db[mountFrame.type][self.name] and 0 or 1)
		self:SetChecked(Valet.db[mountFrame.type][self.name] and 1 or 0)
	end
	local function ShowTooltip(self)
		if not self.name then return end
		GameTooltip:SetOwner(self, 'ANCHOR_RIGHT')
		GameTooltip:SetText('|cffffffff'..self.name)
		GameTooltip:Show()
	end
	local function HideTooltip() GameTooltip:Hide() end

	for i=1,NUMROWS do
		local row = CreateFrame('Frame', nil, mountFrame)
		row:SetHeight(ICONSIZE)
		if i == 1 then row:SetPoint('TOPLEFT', mountFrame, EDGEGAP, -EDGEGAP-ICONSIZE-6)
		else row:SetPoint('TOPLEFT', rows[i-1], 'BOTTOMLEFT', 0, -6) end
		row:SetPoint('RIGHT', -EDGEGAP, 0)
		row.buttons = {}
		rows[i] = row

		for j=1,NUMCOLS do
			local iconbutton = CreateFrame('CheckButton', nil, row)
			if j == 1 then iconbutton:SetPoint('TOPLEFT', row, 'TOPLEFT')
			else iconbutton:SetPoint('LEFT', row.buttons[j-1], 'RIGHT', GAP, 0) end
			iconbutton:SetWidth(ICONSIZE)
			iconbutton:SetHeight(ICONSIZE)

			iconbutton:SetHighlightTexture('Interface\\Buttons\\ButtonHilight-Square')
			iconbutton:SetCheckedTexture('Interface\\Buttons\\UI-Button-Outline')
			local tex = iconbutton:GetCheckedTexture()
			tex:ClearAllPoints()
			tex:SetPoint('CENTER')
			tex:SetWidth(ICONSIZE/37*66) tex:SetHeight(ICONSIZE/37*66)

			iconbutton:SetScript('OnEnter', ShowTooltip)
			iconbutton:SetScript('OnLeave', HideTooltip)
			iconbutton:SetScript('OnClick', OnClick)

			row.buttons[j] = iconbutton
		end
	end

	local offset = 0
	local function Update()
		if not Valet or not Valet.db then return end
		
		if not mountFrame:IsShown() then mountFrame:Show() end
		mountFrameText:SetText('These mounts will be used at random for the %'..mountFrame.type:upper()..'% value.')
		scrollbar:SetMinMaxValues(0, math.max(0, math.ceil(GetNumCompanions('MOUNT')/NUMCOLS - NUMROWS)))

		for i=1,NUMROWS do
			for j=1,NUMCOLS do
				local butt, buttoffset = rows[i].buttons[j], (i+offset-1)*NUMCOLS + j
				local _, _, spellID, tex = GetCompanionInfo('MOUNT', buttoffset)
				local name = spellID and GetSpellInfo(spellID)
				if name and tex then
					butt.name = name
					butt:SetNormalTexture(tex)
					butt:GetNormalTexture():SetDesaturated(Valet.db[mountFrame.type][name] and 0 or 1)
					butt:SetChecked(Valet.db[mountFrame.type][name] and 1 or 0)
					butt:Show()
				else
					butt:Hide()
				end
			end
		end
	end

	local f = scrollbar:GetScript('OnValueChanged')
	scrollbar:SetScript('OnValueChanged', function(self, value, ...)
		offset = math.floor(value)
		Update()
		return f(self, value, ...)
	end)

	Update()
	scrollbar:SetValue(0)
	mountFrame:SetScript('OnShow', Update)
	mountFrame.Update = Update

	----------------------------
	--      Macro panel      --
	----------------------------
	
	macroFrame = CreateFrame('Frame', nil, group)
	macroFrame:SetAllPoints()
	macroFrame:Hide()

	local macroEditbox = CreateFrame('EditBox', nil, macroFrame)
	macroEditbox:SetPoint('TOPLEFT', EDGEGAP/3, -EDGEGAP/3)
	macroEditbox:SetPoint('BOTTOMRIGHT', -EDGEGAP/3, EDGEGAP/3)
	macroEditbox:SetFontObject(GameFontHighlight)
	macroEditbox:SetTextInsets(8,8,8,8)
	macroEditbox:SetMultiLine(true)
	macroEditbox:SetAutoFocus(false)
	macroEditbox:SetText(Valet.db.macro)
	
	local function SaveMacro()
		local newmacro = macroEditbox:GetText()
		if not (newmacro:find('%%GROUND%%') or newmacro:find('%%FLYER%%')) then
			Valet:Print('Macro text must contain the string \'%GROUND%\' or \'%FLYER%\'.')
		else
			Valet.db.macro = newmacro
			Valet:Shuffle()
		end
	end
	
	local saveButton = LibStub('tekKonfig-Button').new(macroEditbox, 'BOTTOMRIGHT', macroEditbox, 'BOTTOMRIGHT', -GAP/2, GAP/2)
	saveButton:SetText('Save')
	saveButton:SetScript('OnClick', SaveMacro)
	saveButton:SetScale(0.8)
	if InCombatLockdown() then saveButton:Disable() end
	
	local resetButton = LibStub('tekKonfig-Button').new(macroEditbox, 'RIGHT', saveButton, 'LEFT', -GAP/2, 0)
	resetButton:SetText('Reset')
	resetButton:SetScript('OnClick', function()
		macroEditbox:SetText(Valet.defaults.macro)
		SaveMacro()
	end)
	resetButton:SetScale(0.8)
	if InCombatLockdown() then resetButton:Disable() end
	
	macroEditbox:SetScript('OnEditFocusLost', SaveMacro)
	macroEditbox:SetScript('OnEscapePressed', macroEditbox.ClearFocus)
	macroEditbox.tiptext = 'Customize the macro text.  Must include the string \'%GROUND%\' or \'%FLYER%\', which is replaced with the respective mount type.  This setting is saved on a per-char basis.'
	macroEditbox:SetScript('OnEnter', macroButton:GetScript('OnEnter'))
	macroEditbox:SetScript('OnLeave', macroButton:GetScript('OnLeave'))
	
	macroFrame:SetScript('OnEvent', function(self, event)
		if event == 'PLAYER_REGEN_DISABLED' then saveButton:Disable() resetButton:Disable()
		else saveButton:Enable() resetButton:Enable() end
	end)
	macroFrame:RegisterEvent('PLAYER_REGEN_ENABLED')
	macroFrame:RegisterEvent('PLAYER_REGEN_DISABLED')
	
	
	frame:SetScript('OnShow', nil)
end)


InterfaceOptions_AddCategory(frame)

LibStub('tekKonfig-AboutPanel').new('Valet', 'Valet')


----------------------------------------
--      Quicklaunch registration      --
----------------------------------------

local LDB = LibStub:GetLibrary('LibDataBroker-1.1', true)
if LDB then LDB:NewDataObject('Valet', {
	type = 'launcher',
	icon = 'Interface\\Icons\\INV_Misc_Food_DimSum',
	OnClick = function() InterfaceOptionsFrame_OpenToCategory(frame) end,
}) end