------------------------------------------------------------------------------
--	Valet -- (C) 2009-2010 Kyahx.  All Rights Reserved
--
--	Most of this code was inspired by (or shamelessly copied from)
--	Tekkub's mods, Kennel and Buffet. All credit to him where credit is due.
------------------------------------------------------------------------------

----------------------
--      Locals      --
----------------------

local defaults = {macro = "#showtooltip\n/use [flyable,nomod:alt] %FLYER%; %GROUND%", flyer = {}, ground = {}}
local dirty, fallback


-----------------------------
--      Event Handler      --
-----------------------------

Valet = CreateFrame('frame')
Valet:SetScript('OnEvent', function(self, event, ...) if self[event] then return self[event](self, ...) end end)
Valet:RegisterEvent('ADDON_LOADED')
function Valet:Print(...) DEFAULT_CHAT_FRAME:AddMessage(string.join(' ', '|cFF33FF99Valet|r:', ...)) end


--------------------
--      Core      --
--------------------

function Valet:ADDON_LOADED(addon)
	if addon ~= 'Valet' then return end
	
	ValetDB = setmetatable(ValetDB or {}, {__index = defaults})
	self.db = ValetDB
	self.defaults = defaults
	
	local metatable = {__index = function(t,i)
		for k,v in pairs(t) do
			if i == v then return k end
		end
	end}
	
	self.db.flyer = setmetatable(self.db.flyer or {}, metatable)
	self.db.ground = setmetatable(self.db.ground or {}, metatable)
	
	self:UnregisterEvent('ADDON_LOADED')
	self.ADDON_LOADED = nil

	if IsLoggedIn() then self:PLAYER_LOGIN() else self:RegisterEvent('PLAYER_LOGIN') end
end


function Valet:PLAYER_LOGIN()
	if GetNumCompanions('MOUNT') < 1 then
		self:RegisterEvent('COMPANION_LEARNED')
		self.COMPANION_LEARNED = self.PLAYER_LOGIN
		return
	end
	
	self:RegisterEvent('COMPANION_UPDATE')
	self:RegisterEvent('PLAYER_REGEN_ENABLED')

	fallback = select(2,GetCompanionInfo('MOUNT', 1))
	self:Shuffle()

	self:UnregisterEvent('PLAYER_LOGIN')
	self.PLAYER_LOGIN = nil
end


function Valet:PLAYER_REGEN_ENABLED()
	if dirty then self:Shuffle() end
end


function Valet:COMPANION_UPDATE(type)
	if type ~= 'MOUNT' then return end
	
	local t = 0
	self:SetScript('OnUpdate',function(_,e)
		t = t+e
		if t > 0.5 then
			if not IsMounted() then
				dirty = true
				if not InCombatLockdown() then self:Shuffle() end
			end
			self:SetScript('OnUpdate', nil)
		end
	end)
end


function Valet:Shuffle()
	local flyer, ground
	
	if self.db.flyer and #self.db.flyer > 0 then flyer = self.db.flyer[random(#self.db.flyer)] end
	if self.db.ground and #self.db.ground > 0 then ground = self.db.ground[random(#self.db.ground)] end

	self:Edit('AutoMount', self.db.macro, flyer or fallback, ground or fallback)
	dirty = false
end


function Valet:Edit(name, substring, flyer, ground)
	local macroid = GetMacroIndexByName(name)
	if not macroid then return end
	
	if flyer then substring = substring:gsub('%%FLYER%%', flyer) end
	if ground then substring = substring:gsub('%%GROUND%%', ground) end
	
	EditMacro(macroid, name, 1, substring, 1)
end
